# Tag 1

## Was ist ein Container?
Container sind leichtgewichtige, isolierte Umgebungen, die Anwendungen und ihre Abhängigkeiten bündeln. Sie funktionieren unabhängig vom Host-System und sorgen für Konsistenz zwischen Entwicklungs-, Test- und Produktionsumgebungen.
## Nutzen von Containern:
**Portabilität** – Container laufen auf jeder Plattform mit Docker-Umgebung.

**Ressourceneffizienz** – Im Gegensatz zu virtuellen Maschinen benötigen sie keine eigene Betriebssysteminstanz.

**Skalierbarkeit** – Anwendungen können schnell gestartet, gestoppt und vervielfältigt werden.

Docker-Container sind besonders nützlich für Microservices, CI/CD-Pipelines, Cloud-Anwendungen und skalierbare Webanwendungen.

## Devops
DevOps ist eine Kombination aus "Development" (Entwicklung) und "Operations" (Betrieb). Es ist eine Sammlung von Praktiken, die darauf abzielt, die Zusammenarbeit zwischen Softwareentwicklern und IT-Operations-Teams zu verbessern.

Ziele von DevOps:

**Schnellere Softwareentwicklung** – Automatisierung von Builds, Tests und Deployments.

**Höhere Qualität** – Durch Continuous Integration (CI) und Continuous Deployment (CD).

**Bessere Zusammenarbeit** – Entwickler und Admins arbeiten eng zusammen.

**Mehr Zuverlässigkeit** – Durch Monitoring, Logging und schnelle Fehlerbehebung.

## Container und Images
Ein Image ist eine unveränderliche Vorlage, die alle notwendigen Dateien, Bibliotheken und Konfigurationen für eine Anwendung enthält. Es dient als Blaupause für Container. Ein Container hingegen ist eine laufende Instanz eines Images – also eine aktive Umgebung, die auf diesem Image basiert. Während Images statisch und wiederverwendbar sind, sind Container dynamisch und können gestartet, gestoppt oder gelöscht werden. Mehrere Container können aus demselben Image erstellt werden, laufen aber unabhängig voneinander.

## Commands

### Container-Verwaltung:

| Beschreibung | Command |
|---|---|
| docker run IMAGE | Startet einen neuen Container aus einem Image |
| docker ps | Zeigt laufende Container |
| docker ps -a | Zeigt alle Container (auch gestoppte) |
| docker rm CONTAINER | Löscht einen gestoppten Container |


### Image-Verwaltung:

| Beschreibung | Command |
|---|---|
| docker images | Listet alle lokalen Images auf |
| docker pull IMAGE | Lädt ein Image aus Docker Hub herunter |
| docker rmi IMAGE | Löscht ein lokales Image |
| docker build -t name:tag PATH | docker image builden |
| docker push name:tag | Image zu Container Registery pushen |


### Netzwerk & Logs:

| Beschreibung | Command |
|---|---|
| docker logs CONTAINER | Zeigt die Logs eines Containers |
| docker exec -it CONTAINER bash | Öffnet eine Shell in einem laufenden Container |
| docker inspect CONTAINER/IMAGE | Zeigt detaillierte Infos zu einem Container/Image |

### Volumes & Daten:

| Beschreibung | Command |
|---|---|
| docker volume ls | Zeigt alle Volumes |
| docker volume rm VOLUME | Löscht ein Volume |


### Docker Compose:

| Beschreibung | Command |
|---|---|
| docker compose up | Services aus lokalem docker-compose.yaml starten |
| docker compose up -d | Services aus lokalem docker-compose.yaml detached starten |
| docker compose down | Services wieder stoppen |
| docker compose logs | Logs anzeigen |
| docker compose ps | Services und deren Status anzeigen |
| docker volume rm VOLUME | Löscht ein Volume |

•	docker compose up → Services aus lokalem docker-compose.yaml starten
•	docker compose down → Services wieder stoppen
•	docker compose logs → Logs anzeigen
•	docker compose ps → Services und deren Status anzeigen
•	docker volume rm VOLUME → Löscht ein Volume
